@@ -5,21 +5,25 @@ import sys
 import tempfile
 import unittest
 
+from typing import List, Tuple
 from itertools import islice
 from pathlib import Path
 from unittest import mock
 
 from bpython import config, repl, cli, autocomplete
-from bpython.test import MagicIterMock, FixLanguageTestCase as TestCase
-from bpython.test import TEST_CONFIG
+from bpython.line import LinePart
+from bpython.test import (
+    MagicIterMock,
+    FixLanguageTestCase as TestCase,
+    TEST_CONFIG,
+)
 
 
 pypy = "PyPy" in sys.version
 
 
 def setup_config(conf):
-    config_struct = config.Struct()
-    config.loadini(config_struct, TEST_CONFIG)
+    config_struct = config.Config(TEST_CONFIG)
     if conf is not None and "autocomplete_mode" in conf:
         config_struct.autocomplete_mode = conf["autocomplete_mode"]
     return config_struct
@@ -35,9 +39,32 @@ class FakeHistory(repl.History):
 
 class FakeRepl(repl.Repl):
     def __init__(self, conf=None):
-        repl.Repl.__init__(self, repl.Interpreter(), setup_config(conf))
-        self.current_line = ""
-        self.cursor_offset = 0
+        super().__init__(repl.Interpreter(), setup_config(conf))
+        self._current_line = ""
+        self._cursor_offset = 0
+
+    def _get_current_line(self) -> str:
+        return self._current_line
+
+    def _set_current_line(self, val: str) -> None:
+        self._current_line = val
+
+    def _get_cursor_offset(self) -> int:
+        return self._cursor_offset
+
+    def _set_cursor_offset(self, val: int) -> None:
+        self._cursor_offset = val
+
+    def getstdout(self) -> str:
+        raise NotImplementedError
+
+    def reprint_line(
+        self, lineno: int, tokens: List[Tuple[repl._TokenType, str]]
+    ) -> None:
+        raise NotImplementedError
+
+    def reevaluate(self):
+        raise NotImplementedError
 
 
 class FakeCliRepl(cli.CLIRepl, FakeRepl):
@@ -100,7 +127,7 @@ class TestMatchesIterator(unittest.TestCase):
 
         newmatches = ["string", "str", "set"]
         completer = mock.Mock()
-        completer.locate.return_value = (0, 1, "s")
+        completer.locate.return_value = LinePart(0, 1, "s")
         self.matches_iterator.update(1, "s", newmatches, completer)
 
         newslice = islice(newmatches, 0, 3)
@@ -109,7 +136,7 @@ class TestMatchesIterator(unittest.TestCase):
 
     def test_cur_line(self):
         completer = mock.Mock()
-        completer.locate.return_value = (
+        completer.locate.return_value = LinePart(
             0,
             self.matches_iterator.orig_cursor_offset,
             self.matches_iterator.orig_line,
@@ -158,7 +185,8 @@ class TestArgspec(unittest.TestCase):
     def test_func_name(self):
         for (line, expected_name) in [
             ("spam(", "spam"),
-            ("spam(map([]", "map"),
+            # map pydoc has no signature in pypy
+            ("spam(any([]", "any") if pypy else ("spam(map([]", "map"),
             ("spam((), ", "spam"),
         ]:
             self.set_input_line(line)
@@ -168,7 +196,8 @@ class TestArgspec(unittest.TestCase):
     def test_func_name_method_issue_479(self):
         for (line, expected_name) in [
             ("o.spam(", "spam"),
-            ("o.spam(map([]", "map"),
+            # map pydoc has no signature in pypy
+            ("o.spam(any([]", "any") if pypy else ("o.spam(map([]", "map"),
             ("o.spam((), ", "spam"),
         ]:
             self.set_input_line(line)
@@ -201,6 +230,7 @@ class TestArgspec(unittest.TestCase):
         # Argument position
         self.assertEqual(self.repl.arg_pos, 1)
 
+    @unittest.skipIf(pypy, "range pydoc has no signature in pypy")
     def test_issue127(self):
         self.set_input_line("x=range(")
         self.assertTrue(self.repl.get_args())
@@ -308,9 +338,14 @@ class TestGetSource(unittest.TestCase):
         self.assert_get_source_error_for_current_function(
             collections.defaultdict.copy, "No source code found for INPUTLINE"
         )
-        self.assert_get_source_error_for_current_function(
-            collections.defaultdict, "could not find class definition"
-        )
+        if sys.version_info[:2] >= (3, 13):
+            self.assert_get_source_error_for_current_function(
+                collections.defaultdict, "source code not available"
+            )
+        else:
+            self.assert_get_source_error_for_current_function(
+                collections.defaultdict, "could not find class definition"
+            )
 
     def test_current_line(self):
         self.repl.interp.locals["a"] = socket.socket
@@ -480,6 +515,34 @@ class TestRepl(unittest.TestCase):
             self.repl.matches_iter.matches, ["abc=", "abd=", "abs("]
         )
 
+    def test_parameter_advanced_on_class(self):
+        self.repl = FakeRepl(
+            {"autocomplete_mode": autocomplete.AutocompleteModes.SIMPLE}
+        )
+        self.set_input_line("TestCls(app")
+
+        code = """
+        import inspect
+
+        class TestCls:
+            # A class with boring __init__ typing
+            def __init__(self, *args, **kwargs):
+                pass
+            # But that uses super exotic typings recognized by inspect.signature
+            __signature__ = inspect.Signature([
+                inspect.Parameter("apple", inspect.Parameter.POSITIONAL_ONLY),
+                inspect.Parameter("apple2", inspect.Parameter.KEYWORD_ONLY),
+                inspect.Parameter("pinetree", inspect.Parameter.KEYWORD_ONLY),
+            ])
+        """
+        for line in code.split("\n"):
+            print(line[8:])
+            self.repl.push(line[8:])
+
+        self.assertTrue(self.repl.complete())
+        self.assertTrue(hasattr(self.repl.matches_iter, "matches"))
+        self.assertEqual(self.repl.matches_iter.matches, ["apple2=", "apple="])
+
 
 class TestCliRepl(unittest.TestCase):
     def setUp(self):
